/*
 * Copyright Fraunhofer ISE, energy & meteo Systems GmbH, and other contributors 2011
 *
 * This file is part of openIEC61850.
 * For more information visit http://www.openmuc.org 
 *
 * openIEC61850 is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * openIEC61850 is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with openIEC61850.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
package org.openiec61850.common.model;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

import org.openiec61850.common.model.report.ReportControlBlock;

/**
 * 
 * @author Michael Zillgith
 * @author Stefan Feuerhahn
 * @author Tobias Weidelt
 * 
 */
final public class Server extends ModelNode {

	private String authentication = null;
	private Map<String, ReportControlBlock> rcbs = new HashMap<String, ReportControlBlock>();
	private Map<String, DataSet> dataSets = new HashMap<String, DataSet>();

	public Server(List<LogicalDevice> logicalDevices) {
		children = new LinkedHashMap<String, ModelNode>();
		objectReference = null;
		for (LogicalDevice logicalDevice : logicalDevices) {
			this.children.put(logicalDevice.getReference().getName(), logicalDevice);
		}
		populateRcbDS();
	}

	// TODO authentication necessary?
	public String getAuthentication() {
		return authentication;
	}

	public void setAuthentication(String authentication) {
		this.authentication = authentication;
	}

	@Override
	public Server copy() {
		List<LogicalDevice> childCopies = new ArrayList<LogicalDevice>(children.size());
		for (ModelNode childNode : children.values()) {
			childCopies.add((LogicalDevice) childNode.copy());
		}

		return new Server(childCopies);

	}

	public DataSet getDataSet(String dataSetRef) {

		return dataSets.get(dataSetRef);
	}

	public Collection<DataSet> getDataSets() {

		return dataSets.values();
	}

	public ReportControlBlock getRCB(String rcbRef) {
		return rcbs.get(rcbRef);
	}

	public Collection<ReportControlBlock> getRCBs() {
		return rcbs.values();
	}

	public void setDataSet(DataSet ds) {
		dataSets.put(ds.getReference().toString(), ds);
	}

	public void setRCBs(ReportControlBlock rcb) {
		rcbs.put(rcb.getReference().toString(), rcb);
	}

	public void deleteDataSet(DataSet ds) {
		dataSets.remove(ds.getReference().toString());
	}

	@Override
	public String toString() {
		return "Server";
	}

	private void populateRcbDS() {
		for (ModelNode ld : children.values()) {
			for (ModelNode ln : ld.getChildren()) {
				for (DataSet ds : ((LogicalNode) ln).getDataSets()) {
					dataSets.put(ds.getReference().toString(), ds);
				}
				for (ReportControlBlock rcb : ((LogicalNode) ln).getReportControlBlocks()) {
					rcbs.put(rcb.getReference().toString(), rcb);
				}
			}
		}
	}
}
