package org.openiec61850.sample;

/*
 * Copyright Fraunhofer ISE, energy & meteo Systems GmbH, and other contributors 2011
 *
 * This file is part of openIEC61850.
 * For more information visit http://www.openmuc.org 
 *
 * openIEC61850 is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * openIEC61850 is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with openIEC61850.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

import java.util.List;
import java.util.Map;

import org.openiec61850.client.ClientACSI;
import org.openiec61850.common.ServiceError;
import org.openiec61850.common.model.BasicDataAttribute;
import org.openiec61850.common.model.DataObject;
import org.openiec61850.common.model.FunctionalConstraint;
import org.openiec61850.common.model.LogicalNode;
import org.openiec61850.common.model.ModelNode;
import org.openiec61850.common.model.Server;
import org.openiec61850.common.model.datypes.basic.BOOLEAN;
import org.openiec61850.common.model.datypes.basic.VISIBLE_STRING_65;
import org.openiec61850.common.model.report.Report;
import org.openiec61850.common.model.report.ReportControlBlock;
import org.openiec61850.scsm.mms.client.MmsScsmClientSAP;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * 
 * @author Stefan Feuerhahn
 * 
 *         Note that this class is for demonstration purposes only. Since the
 *         openIEC61850 server does not support reporting yet it cannot be run
 *         in combination with the SampleServer.
 * 
 */
public class SampleReportClient extends Thread {

	private static Logger logger = LoggerFactory.getLogger(SampleReportClient.class);

	static ClientACSI mmsAssociation = null;

	public static void main(String[] args) throws InterruptedException, ServiceError {

		if (args.length != 2) {
			System.out.println("usage: org.openiec61850.sample.SampleMMSClient <host> <port>");
			return;
		}

		String serverAddress = args[0];
		int serverPort = Integer.parseInt(args[1]);

		logger.debug("Using MMS");
		MmsScsmClientSAP clientSAP = new MmsScsmClientSAP();

		// optionally you can set the remote and local Transport Selectors and
		// the Maximum TPDUSize:
		clientSAP.acseSAP.tSAP.tSelRemote = new byte[] { 0, 1 };
		clientSAP.acseSAP.tSAP.tSelLocal = new byte[] { 0, 0 };
		clientSAP.acseSAP.tSAP.setMaxTPDUSizeParam(10);

		logger.debug("Associating with server " + serverAddress + " on port " + serverPort);
		// ClientACSI mmsAssociation = clientSAP.associate(serverAddress + ":" +
		// serverPort, "", 60000);

		mmsAssociation = clientSAP.associate(serverAddress + ":" + serverPort, null, 60000);

		Server server = mmsAssociation.retrieveModel();

		printSortedByFC(server);

		List<BasicDataAttribute> basicDataAttributes = server.getBasicDataAttributes();

		System.out.println("numBDAs: " + basicDataAttributes.size());

		for (BasicDataAttribute bda : basicDataAttributes) {
			// mmsAssociation.getDataValues(bda);
			// mmsAssociation.setDataValues(bda);
		}

		SampleReportClient reportHandlerThread = new SampleReportClient();
		reportHandlerThread.start();

		// you have to replace the reference with the correct reference of the
		// URCB that you want:
		ReportControlBlock urcb = server.getRCB("SomeLDName/LLN0.exampleURCB");
		if (urcb == null) {
			logger.error("ReportControlBlock not found");
		}
		else {
			mmsAssociation.getRCBValues(urcb);
			logger.debug("urcb name: " + urcb.getName());
			logger.debug("urcb name: " + urcb.getNodeName());
			logger.debug("RptID: " + ((VISIBLE_STRING_65) urcb.getChild("RptID")).getValue());
			((BOOLEAN) urcb.getChild("RptEna")).setValue(true);
			((BOOLEAN) urcb.getChild("GI")).setValue(true);
			mmsAssociation.setRCBValues(urcb);
		}
		logger.debug("Client is done. Will quit.");

	}

	public void run() {
		Report report = null;
		try {
			report = mmsAssociation.getReport();
		} catch (ServiceError e) {
			e.printStackTrace();
		}

		logger.debug("got report with dataset ref: " + report.getDataSet().getReference());
		// do something with the report

	}

	public static void printSortedByFC(Server server) {
		logger.debug(server.toString());
		for (ModelNode ld : server) {
			logger.debug(ld.toString());
			for (ModelNode ln : ld) {
				logger.debug(ln.toString());
				LogicalNode logicalNode = (LogicalNode) ln;
				Map<FunctionalConstraint, Map<String, DataObject>> fcChildrenMapMap = logicalNode.getFCChildren();

				for (Map<String, DataObject> fcChildrenMap : fcChildrenMapMap.values()) {
					for (ModelNode fcChild : fcChildrenMap.values()) {
						printRecursive(fcChild);
					}
				}
			}
		}
	}

	private static void printRecursive(ModelNode modelNode) {
		logger.debug(modelNode.toString());
		if (modelNode.getChildren() != null) {
			for (ModelNode modelNodeChild : modelNode.getChildren()) {
				printRecursive(modelNodeChild);
			}
		}
	}

}
